"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const util_1 = require("util");
const worker_threads_1 = require("worker_threads");
const logs_1 = require("./logs");
const util_2 = require("./util");
/**
 * Clase para la definicion y manejo del microservicio.
 */
class MsManager {
    /**
     * Metodo que ejecuta una clase
     * @param Target Clase a ejecutar
     */
    static execute(Target) {
        // Crear la instancia.
        const inst = new Target(Reflect.getMetadata('definition:conf', Target));
        // Ciclo infinito para no Terminar el proceso (Thread)
        if (!worker_threads_1.isMainThread) {
            setInterval(() => {
                // NO SE REQUIERE EJECUTAR NADA.
            }, 500);
        }
    }
    /**
     * Metodo para el el registro del micro servicio
     * @param Conf Configuracion del microservicio
     */
    static Define(Conf) {
        // Regresar la funcion decoradora
        return (Target) => {
            // Grabar la Configuracion
            Reflect.defineMetadata('definition:conf', Conf, Target);
            // Auto ejecutar la claxe
            MsManager.execute(Target);
            // Regresar el objeto inicial
            return Target;
        };
    }
    /**
     * @public
     * @static
     */
    static List(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:list', true, forks);
    }
    /**
     * @public
     * @static
     */
    static Create(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:create', true, forks);
    }
    /**
     * @public
     * @static
     */
    static Get(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:get', true, forks);
    }
    /**
     * @public
     * @static
     */
    static Update(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:update', true, forks);
    }
    /**
     * @public
     * @static
     */
    static Delete(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:delete', true, forks);
    }
    /**
     * @public
     * @static
     */
    static Listener(forks = null) {
        // Indicar el tipo
        return MsManager.defineElement('definition:listener', true, forks);
    }
    /**
     * Metodo para definir la configuracion de los Forks
     */
    static Fork() {
        // Forks a los que se tiene acceso.
        return MsManager.defineElement('definition:fork', true);
    }
    static Errors() {
        // Indicar el tipo
        return MsManager.defineElement('definition:errors', true);
    }
    /**
     * Metodo para el registro Raw de un elemento de clase
     * @param conf Configuracion del elemento
     * @param type Tipo del elemento
     * @param isFunction Indicar si se espera una funcion
     * @param confMethod Configuracion del elemento decorado
     */
    static defineElement(Type, isFunction = false, confMethod = null) {
        // Regresar el objecto la funcion decoradora
        return (target, key, descriptor) => {
            const Conf = {
                key,
                returntype: Reflect.getMetadata('design:returntype', target, key),
                type: Reflect.getMetadata('design:type', target, key),
            };
            // Validar si es necesario que se haya proporcionado una funcion
            if (isFunction) {
                if (!util_2.default.isFunction(target[key])) {
                    // Indicar que ocurrio un error
                    MsManager.logs.error(`Se requiere que [${key}] sea una funcion.`);
                    // Salir e indicar error para que no se incie el servicio
                    process.exit(-1);
                }
            }
            // Agregar la meta data del evento
            Reflect.defineMetadata(Type, Conf, target);
            // Validar si es requerido almacenar los datos de configuracion
            if (!util_1.isNull(confMethod)) {
                // Grabar los datos.
                Reflect.defineMetadata(`${Type}:conf`, confMethod, target);
            }
            // Regresar la funcion original
            return descriptor;
        };
    }
    /**
     * Metodo para recuperar toda la Meta data de un target
     * @param Target
     */
    static getAllMetaData(Target) {
        // Recuperar todas las Keys
        const Keys = Reflect.getMetadataKeys(Target);
        // Objeto donde se almacenaran los datos.
        const resulObj = [];
        // Recorrer y recuperar todos los datos
        for (const k of Keys) {
            // No recuperar los datos de configuracion
            if (k !== 'definition:conf') {
                // Almacenar la clave
                resulObj.push(Reflect.getMetadata(k, Target));
            }
        }
        // Regresar el resultado
        return resulObj;
    }
    /**
     * metodo para recuperar los datos desde el contexto
     * de la aplicacion.
     * @param name Nombre del elemento
     */
    static ctx(name) {
        return (target, propertyKey, parameterIndex) => {
            // Parametros
            let col = [];
            // Verificar si ya existe uno previo
            if (Reflect.hasMetadata(`${propertyKey}:conf`, target)) {
                // Recuperar el valor
                col = Reflect.getMetadata(`${propertyKey}:conf`, target);
            }
            // Agregar el nuevo registro
            col.unshift(name);
            // Grabar los datos
            Reflect.defineMetadata(`${propertyKey}:conf`, col, target);
        };
    }
}
exports.MsManager = MsManager;
/**
 * Manejador de logs
 * @protected
 */
MsManager.logs = logs_1.default;
